@@ -51,6 +51,7 @@
  *
  */
 
+#include <config.h>
 #include <glib.h>
 #include <string.h>
 #include <unistd.h>
@@ -65,7 +66,7 @@
 
 #define VERSION_MAJOR 0
 #define VERSION_MINOR 1
-#define VERSION "0.4"
+#define PVERSION "0.4"
 
 #ifdef WIN32
 #undef WITH_THREAD /* Thread support locks up xchat on Win32. */
@@ -220,7 +221,7 @@ static PyObject *Context_FromServerAndChannel(char *server, char *channel);
 
 static PyObject *Plugin_New(char *filename, PyMethodDef *xchat_methods,
 			    PyObject *xcoobj);
-static PyObject *Plugin_GetCurrent();
+static PyObject *Plugin_GetCurrent(void);
 static PluginObject *Plugin_ByString(char *str);
 static Hook *Plugin_AddHook(int type, PyObject *plugin, PyObject *callback,
 			    PyObject *userdata, void *data);
@@ -257,6 +258,10 @@ static void Command_PyReload(char *name);
 static void Command_PyAbout();
 static int Command_Py(char *word[], char *word_eol[], void *userdata);
 
+void xchat_plugin_get_info(char **name, char **desc, char **version, void **reserved);
+int xchat_plugin_init(xchat_plugin *plugin_handle, char **plugin_name, char **plugin_desc, char **plugin_version, char *arg);
+int xchat_plugin_deinit(void);
+
 /* ===================================================================== */
 /* Static declarations and definitions */
 
@@ -290,7 +295,7 @@ Usage: /PY LOAD   <filename>\n\
 
 static char *about = "\
 \n\
-X-Chat Python Interface " VERSION "\n\
+X-Chat Python Interface " PVERSION "\n\
 \n\
 Copyright (c) 2002-2003  Gustavo Niemeyer <niemeyer@conectiva.com>\n\
 \n";
@@ -1163,7 +1168,7 @@ error:
 }
 
 static PyObject *
-Plugin_GetCurrent()
+Plugin_GetCurrent(void)
 {
 	PyObject *plugin;
 	plugin = PySys_GetObject("__plugin__");
@@ -2025,7 +2030,7 @@ void
 xchat_plugin_get_info(char **name, char **desc, char **version, void **reserved)
 {
 	*name = "Python";
-	*version = VERSION;
+	*version = PVERSION;
 	*desc = "Python scripting interface";
    if (reserved)
       *reserved = NULL;
@@ -2053,7 +2058,7 @@ xchat_plugin_init(xchat_plugin *plugin_handle,
 	initialized = 1;
 
 	*plugin_name = "Python";
-	*plugin_version = VERSION;
+	*plugin_version = PVERSION;
 	*plugin_desc = "Python scripting interface";
 
 	/* Initialize python. */
@@ -2111,7 +2116,7 @@ xchat_plugin_init(xchat_plugin *plugin_handle,
 }
 
 int
-xchat_plugin_deinit()
+xchat_plugin_deinit(void)
 {
 	GSList *list;
 
