@@ -106,8 +106,7 @@ random_line (char *file_name)
 
 	/* go down a random number */
 	rewind (fh);
-	srand (time (0));
-	ran = rand () % lines;
+	ran = RAND_INT (lines);
 	do
 	{
 		fgets (buf, sizeof (buf), fh);
@@ -392,7 +391,7 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 	char username[64], fullhost[128], domain[128], *mode, *p2;
 	server *serv = sess->server;
 
-	user = find_name (sess, mask);
+	user = userlist_find (sess, mask);
 	if (user && user->hostname)  /* it's a nickname, let's find a proper ban mask */
 	{
 		if (deop)
@@ -1275,7 +1274,7 @@ cmd_dns (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	{
 		if (strchr (nick, '.') == NULL)
 		{
-			user = find_name (sess, nick);
+			user = userlist_find (sess, nick);
 			if (user && user->hostname)
 			{
 				do_dns (sess, user->nick, user->hostname);
@@ -1498,7 +1497,7 @@ cont:				esc = FALSE;
 					k = 0;
 				} else
 				{
-					if (isdigit (buf[i]) && k < (sizeof (numb) - 1))
+					if (isdigit ((unsigned char) buf[i]) && k < (sizeof (numb) - 1))
 					{
 						numb[k] = buf[i];
 						k++;
@@ -1559,7 +1558,13 @@ exec_data (GIOChannel *source, GIOCondition condition, struct nbexec *s)
 			buf[len] = '\0';
 			exec_handle_colors(buf, len);
 			if (s->tochannel)
+			{
+				/* must turn off auto-completion temporarily */
+				unsigned int old = prefs.nickcompletion;
+				prefs.nickcompletion = 0;
 				handle_multiline (s->sess, buf, FALSE, TRUE);
+				prefs.nickcompletion = old;
+			}
 			else
 				PrintText (s->sess, buf);
 		}
@@ -1639,7 +1644,7 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			if (access ("/bin/sh", X_OK) != 0)
 			{
-				fe_message (_("I need /bin/sh to run!\n"), FALSE);
+				fe_message (_("I need /bin/sh to run!\n"), FE_MSG_ERROR);
 				return TRUE;
 			}
 		}
@@ -1833,6 +1838,18 @@ cmd_getstr (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+cmd_ghost (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (word[3][0])
+	{
+		sess->server->p_ns_ghost (sess->server, word[2], word[3]);
+		return TRUE;
+	}
+
+	return FALSE; 
+}
+
 static int
 cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1862,7 +1879,7 @@ cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			return FALSE;
 	} else if (!strcasecmp (word[2], "MSGBOX"))
 	{
-		fe_message (word[3], FALSE);
+		fe_message (word[3], FE_MSG_INFO);
 		return TRUE;
 	} else
 	{
@@ -1899,7 +1916,7 @@ show_help_line (session *sess, help_list *hl, char *name, char *usage)
 	p = name;
 	while (*p)
 	{
-		hl->buf[len] = toupper (*p);
+		hl->buf[len] = toupper ((unsigned char) *p);
 		len++;
 		p++;
 	}
@@ -2006,7 +2023,7 @@ cmd_id (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	if (word[2][0])
 	{
-		sess->server->p_nickserv (sess->server, word[2]);
+		sess->server->p_ns_identify (sess->server, word[2]);
 		return TRUE;
 	}
 
@@ -2134,9 +2151,9 @@ cmd_kickban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	{
 		/* if the reason is a 1 digit number, treat it as a bantype */
 
-		user = find_name (sess, nick);
+		user = userlist_find (sess, nick);
 
-		if (isdigit (reason[0]) && reason[1] == 0)
+		if (isdigit ((unsigned char) reason[0]) && reason[1] == 0)
 		{
 			ban (sess, tbuf, nick, reason, (user && user->op));
 			reason[0] = 0;
@@ -2292,7 +2309,7 @@ cmd_me (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (dcc_write_chat (sess->channel, tbuf))
 	{
 		/* print it to screen */
-		inbound_action (sess, sess->channel, sess->server->nick, act, TRUE);
+		inbound_action (sess, sess->channel, sess->server->nick, act, TRUE, FALSE);
 	} else
 	{
 		/* DCC CHAT failed, try through server */
@@ -2300,7 +2317,7 @@ cmd_me (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			sess->server->p_action (sess->server, sess->channel, act);
 			/* print it to screen */
-			inbound_action (sess, sess->channel, sess->server->nick, act, TRUE);
+			inbound_action (sess, sess->channel, sess->server->nick, act, TRUE, FALSE);
 		} else
 		{
 			notc_msg (sess);
@@ -2437,6 +2454,12 @@ static int
 cmd_newserver (struct session *sess, char *tbuf, char *word[],
 					char *word_eol[])
 {
+	if (strcmp (word[2], "-noconnect") == 0)
+	{
+		new_ircwindow (NULL, word[3], SESS_SERVER, 0);
+		return TRUE;
+	}
+	
 	sess = new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 	cmd_server (sess, tbuf, word, word_eol);
 	return TRUE;
@@ -2642,6 +2665,39 @@ cmd_say (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static int
+cmd_send (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	guint32 addr;
+	socklen_t len;
+	struct sockaddr_in SAddr;
+
+	if (!word[2][0])
+		return FALSE;
+
+	addr = dcc_get_my_address ();
+	if (addr == 0)
+	{
+		/* use the one from our connected server socket */
+		memset (&SAddr, 0, sizeof (struct sockaddr_in));
+		len = sizeof (SAddr);
+		getsockname (sess->server->sok, (struct sockaddr *) &SAddr, &len);
+		addr = SAddr.sin_addr.s_addr;
+	}
+	addr = ntohl (addr);
+
+	if ((addr & 0xffff0000) == 0xc0a80000 ||	/* 192.168.x.x */
+		 (addr & 0xff000000) == 0x0a000000)		/* 10.x.x.x */
+		/* we got a private net address, let's PSEND or it'll fail */
+		snprintf (tbuf, 512, "DCC PSEND %s", word_eol[2]);
+	else
+		snprintf (tbuf, 512, "DCC SEND %s", word_eol[2]);
+
+	handle_command (sess, tbuf, FALSE);
+
+	return TRUE;
+}
+
 static int
 cmd_setcursor (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -2684,18 +2740,63 @@ cmd_settext (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static int
+parse_irc_url (char *url, char *server_name[], char *port[], char *channel[], int *use_ssl)
+{
+	char *co;
+#ifdef USE_OPENSSL
+	if (strncasecmp ("ircs://", url, 7) == 0)
+	{
+		*use_ssl = TRUE;
+		*server_name = url + 7;
+		goto urlserv;
+	}
+#endif
+
+	if (strncasecmp ("irc://", url, 6) == 0)
+	{
+		*server_name = url + 6;
+#ifdef USE_OPENSSL
+urlserv:
+#endif
+		/* check for port */
+		co = strchr (*server_name, ':');
+		if (co)
+		{
+			*port = co + 1;
+			*co = 0;
+		} else
+			co = *server_name;
+		/* check for channel - mirc style */
+		co = strchr (co + 1, '/');
+		if (co)
+		{
+			*co = 0;
+			co++;
+			if (*co == '#')
+				*channel = co+1;
+			else
+				*channel = co;
+			
+		}
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static int
 cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	int offset = 0;
-	char *server_name;
-	char *port;
-	char *pass;
-	char *co;
-	server *serv = sess->server;
-#ifdef USE_OPENSSL
+	char *server_name = NULL;
+	char *port = NULL;
+	char *pass = NULL;
+	char *channel = NULL;
 	int use_ssl = FALSE;
+	int is_url = TRUE;
+	server *serv = sess->server;
 
+#ifdef USE_OPENSSL
 	/* BitchX uses -ssl, mIRC uses -e, let's support both */
 	if (strcmp (word[2], "-ssl") == 0 || strcmp (word[2], "-e") == 0)
 	{
@@ -2704,10 +2805,19 @@ cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 #endif
 
-	server_name = word[2 + offset];
-	port = word[3 + offset];
-	pass = word[4 + offset];
-
+	if (!parse_irc_url (word[2 + offset], &server_name, &port, &channel, &use_ssl))
+	{
+		is_url = FALSE;
+		server_name = word[2 + offset];
+	}
+	if (port)
+		pass = word[3 + offset];
+	else
+	{
+		port = word[3 + offset];
+		pass = word[4 + offset];
+	}
+	
 	if (!(*server_name))
 		return FALSE;
 
@@ -2717,45 +2827,10 @@ cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (strncasecmp (word_eol[1], "SERVCHAN ", 9))
 		sess->willjoinchannel[0] = 0;
 
-#ifdef USE_OPENSSL
-	if (strncasecmp ("ircs://", server_name, 7) == 0)
+	if (channel)
 	{
-		use_ssl = TRUE;
-		server_name += 7;
-		goto urlserv;
-	}
-#endif
-
-	if (strncasecmp ("irc://", server_name, 6) == 0)
-	{
-		server_name += 6;
-#ifdef USE_OPENSSL
-urlserv:
-#endif
-		/* check for port */
-		co = strchr (server_name, ':');
-		if (co)
-		{
-			port = co + 1;
-			*co = 0;
-			pass = word[3 + offset];
-		} else
-			co = server_name;
-		/* check for channel - mirc style */
-		co = strchr (co + 1, '/');
-		if (co)
-		{
-			*co = 0;
-			co++;
-			if (*co == '#')
-			{
-				safe_strcpy (sess->willjoinchannel, co, CHANLEN);
-			} else
-			{
-				sess->willjoinchannel[0] = '#';
-				safe_strcpy ((sess->willjoinchannel + 1), co, (CHANLEN - 1));
-			}
-		}
+		sess->willjoinchannel[0] = '#';
+		safe_strcpy ((sess->willjoinchannel + 1), channel, (CHANLEN - 1));
 	}
 
 	/* support +7000 style ports like mIRC */
@@ -2777,7 +2852,7 @@ urlserv:
 #endif
 
 	/* try to connect by Network name */
-	if (servlist_connect_by_netname (sess, server_name))
+	if (servlist_connect_by_netname (sess, server_name, !is_url))
 		return TRUE;
 
 	if (*port)
@@ -2873,12 +2948,68 @@ cmd_unload (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static server *
+find_server_from_net (void *net)
+{
+	GSList *list = serv_list;
+	server *serv;
+
+	while (list)
+	{
+		serv = list->data;
+		if (serv->network == net && serv->connected)
+			return serv;
+		list = list->next;
+	}
+
+	return NULL;
+}
+
 static int
 cmd_url (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	if (word[2][0])
 	{
-		fe_open_url (word[2]);
+		char *server_name = NULL;
+		char *port = NULL;
+		char *channel = NULL;
+		char *url = g_strdup (word[2]);
+		int use_ssl = FALSE;
+		void *net;
+		server *serv;
+
+		if (parse_irc_url (url, &server_name, &port, &channel, &use_ssl))
+		{
+			/* maybe we're already connected to this net */
+
+			/* check for "FreeNode" */
+			net = servlist_net_find (server_name, NULL, strcasecmp);
+			/* check for "irc.eu.freenode.net" */
+			if (!net)
+				net = servlist_net_find_from_server (server_name);
+
+			if (net)
+			{
+				/* found the network, but are we connected? */
+				serv = find_server_from_net (net);
+				if (serv)
+				{
+					/* already connected, JOIN only. FIXME: support keys? */
+					tbuf[0] = '#';
+					/* tbuf is 4kb */
+					safe_strcpy ((tbuf + 1), channel, 256);
+					serv->p_join (serv, tbuf, "");
+					g_free (url);
+					return TRUE;
+				}
+			}
+
+			/* not connected to this net, open new window */
+			cmd_newserver (sess, tbuf, word, word_eol);
+
+		} else
+			fe_open_url (word[2]);
+		g_free (url);
 		return TRUE;
 	}
 
@@ -3106,6 +3237,7 @@ const struct commands xc_cmds[] = {
 	 N_("GATE <host> [<port>], proxies through a host, port defaults to 23")},
 	{"GETINT", cmd_getint, 0, 0, 1, "GETINT <default> <command> <prompt>"},
 	{"GETSTR", cmd_getstr, 0, 0, 1, "GETSTR <default> <command> <prompt>"},
+	{"GHOST", cmd_ghost, 1, 0, 1, N_("GHOST <nick> <password>, Kills a ghosted nickname")},
 	{"GUI", cmd_gui, 0, 0, 1, "GUI [SHOW|HIDE|FOCUS|FLASH|ICONIFY|MENU TOGGLE|COLOR <n>|MSGBOX <text>]"},
 	{"HELP", cmd_help, 0, 0, 1, 0},
 	{"HOP", cmd_hop, 1, 1, 1,
@@ -3151,7 +3283,7 @@ const struct commands xc_cmds[] = {
 	 N_("NAMES, Lists the nicks on the current channel")},
 	{"NCTCP", cmd_nctcp, 1, 0, 1,
 	 N_("NCTCP <nick> <message>, Sends a CTCP notice")},
-	{"NEWSERVER", cmd_newserver, 0, 0, 1, N_("NEWSERVER <hostname> [<port>]")},
+	{"NEWSERVER", cmd_newserver, 0, 0, 1, N_("NEWSERVER [-noconnect] <hostname> [<port>]")},
 	{"NICK", cmd_nick, 0, 0, 1, N_("NICK <nickname>, sets your nick")},
 
 	{"NOTICE", cmd_notice, 1, 0, 1,
@@ -3181,6 +3313,7 @@ const struct commands xc_cmds[] = {
 
 	{"SAY", cmd_say, 0, 0, 1,
 	 N_("SAY <text>, sends the text to the object in the current window")},
+	{"SEND", cmd_send, 0, 0, 1, N_("SEND <nick> [<file>]")},
 #ifdef USE_OPENSSL
 	{"SERVCHAN", cmd_servchan, 0, 0, 1,
 	 N_("SERVCHAN [-ssl] <host> <port> <channel>, connects and joins a channel")},
@@ -3287,9 +3420,9 @@ auto_insert (char *dest, int destlen, unsigned char *src, char *word[],
 	{
 		if (src[0] == '%' || src[0] == '&')
 		{
-			if (isdigit (src[1]))
+			if (isdigit ((unsigned char) src[1]))
 			{
-				if (isdigit (src[2]) && isdigit (src[3]))
+				if (isdigit ((unsigned char) src[2]) && isdigit ((unsigned char) src[3]))
 				{
 					buf[0] = src[1];
 					buf[1] = src[2];
@@ -3442,8 +3575,8 @@ check_special_chars (char *cmd, int do_ascii) /* check for %X */
 				occur++;
 				if (	do_ascii &&
 						j + 3 < len &&
-						(isdigit (cmd[j + 1]) && isdigit (cmd[j + 2]) &&
-						isdigit (cmd[j + 3])))
+						(isdigit ((unsigned char) cmd[j + 1]) && isdigit ((unsigned char) cmd[j + 2]) &&
+						isdigit ((unsigned char) cmd[j + 3])))
 				{
 					tbuf[0] = cmd[j + 1];
 					tbuf[1] = cmd[j + 2];
@@ -3477,6 +3610,9 @@ check_special_chars (char *cmd, int do_ascii) /* check for %X */
 					case 'O':
 						buf[i] = '\017';
 						break;
+					case 'H':	/* CL: invisible text code */
+						buf[i] = '\001';
+						break;
 					case '%':
 						buf[i] = '%';
 						break;
@@ -3746,7 +3882,7 @@ handle_command (session *sess, char *cmd, int check_spch)
 
 	if (command_level > 99)
 	{
-		fe_message (_("Too many recursive usercommands, aborting."), FALSE);
+		fe_message (_("Too many recursive usercommands, aborting."), FE_MSG_ERROR);
 		return TRUE;
 	}
 	command_level++;
