@@ -1,7 +1,7 @@
 /*
  * conversation-panel.c - Widget encapsulating the conversation panel
  *
- * Copyright (C) 2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -298,9 +298,32 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 		return;
 
 	if (event->button == 1) {
-		int type = conversation_panel_check_word (xtext, word, strlen (word));
-		if (type == WORD_URL || type == WORD_HOST) {
-			fe_open_url (word);
+		switch (conversation_panel_check_word (xtext, word, strlen (word))) {
+		case WORD_URL:
+		case WORD_HOST:
+			{
+				char *command;
+				command = g_strdup_printf ("URL %s", word);
+				handle_command (gui.current_session, command, 1);
+				g_free (command);
+			}
+			break;
+		case WORD_NICK:
+			{
+				char *command;
+				command = g_strdup_printf ("QUERY %s", word);
+				handle_command (gui.current_session, command, 1);
+				g_free (command);
+			}
+			break;
+		case WORD_CHANNEL:
+			{
+				char *command;
+				command = g_strdup_printf ("JOIN %s", word);
+				handle_command (gui.current_session, command, 1);
+				g_free (command);
+			}
+			break;
 		}
 	}
 	if (event->button == 3) {
@@ -448,7 +471,10 @@ timestamps_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, xtext
 static void
 open_url (GtkAction *action, ConversationPanel *panel)
 {
-	fe_open_url (panel->priv->selected_word);
+	char *command;
+	command = g_strdup_printf ("URL %s", panel->priv->selected_word);
+	handle_command (gui.current_session, command, 1);
+	g_free (command);
 }
 
 static void
@@ -762,7 +788,7 @@ conversation_panel_save_current (ConversationPanel *panel)
 
 	time (&dtime);
 	localtime_r (&dtime, &date);
-	strftime (dates, 32, "%F-%R", &date);
+	strftime (dates, 32, "%F-%Hh%M", &date);
 
 	default_filename = g_strdup_printf ("%s-%s.log", panel->priv->current->channel, dates);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_chooser), default_filename);
