@@ -1,7 +1,7 @@
 /*
  * fe-gnome.c - main frontend implementation
  *
- * Copyright (C) 2004-2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,28 +19,35 @@
  *
  */
 
+#include <config.h>
+#include <glib/gi18n.h>
+#include <string.h>
 #include <glib.h>
 #include <gnome.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <config.h>
+#include <gconf/gconf-client.h>
+#include "gui.h"
+#include "navigation-tree.h"
+#include "main-window.h"
+#include "userlist-gui.h"
+#include "preferences.h"
+#include "setup-dialog.h"
+
+#include "conversation-panel.h"
+#include "status-bar.h"
+#include "topic-label.h"
+
+#include "palette.h"
+#include "preferences-page-plugins.h"
+#include "channel-list.h"
+#include "util.h"
+#include "plugins.h"
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
 #include "../common/servlist.h"
 #include "../common/fe.h"
 #include "../common/util.h"
 #include "../common/cfgfiles.h"
-#include "gui.h"
-#include "navigation-tree.h"
-#include "textgui.h"
-#include "main-window.h"
-#include "userlist-gui.h"
-#include "preferences.h"
-#include "setup-dialog.h"
-#include "palette.h"
-#include "preferences-page-plugins.h"
-#include "channel-list.h"
-#include "util.h"
-#include "plugins.h"
 
 static gboolean opt_version = FALSE;
 static gboolean opt_noauto = FALSE;
@@ -48,25 +55,47 @@ static gboolean opt_noplugins = FALSE;
 static gchar *opt_cfgdir = NULL;
 
 static GOptionEntry entries[] = {
-	{"cfgdir",     'd', 0, G_OPTION_ARG_FILENAME, &opt_cfgdir,    "Use directory instead of the default config dir", "directory"},
-	{"no-auto",    'a', 0, G_OPTION_ARG_NONE,     &opt_noauto,    "Don't auto-connect to servers",                   NULL},
-	{"no-plugins", 'n', 0, G_OPTION_ARG_NONE,     &opt_noplugins, "Don't auto-load plugins",                         NULL},
-	{"version",    'v', 0, G_OPTION_ARG_NONE,     &opt_version,   "Show version information",                        NULL},
+	{"cfgdir",     'd', 0, G_OPTION_ARG_FILENAME, &opt_cfgdir,           N_("Use directory instead of the default config dir"), "directory"},
+	{"no-auto",    'a', 0, G_OPTION_ARG_NONE,     &arg_dont_autoconnect, N_("Don't auto-connect to servers"),                   NULL},
+	{"no-plugins", 'n', 0, G_OPTION_ARG_NONE,     &opt_noplugins,        N_("Don't auto-load plugins"),                         NULL},
+	{"url",        'u', 0, G_OPTION_ARG_STRING,   &arg_url,              N_("Open an irc:// url"),                              "irc://server:port/channel"},
+	{"version",    'v', 0, G_OPTION_ARG_NONE,     &opt_version,          N_("Show version information"),                        NULL},
+	{ NULL }
 };
 
+void joind (int action, struct server *serv);
+
 int
 fe_args (int argc, char *argv[])
 {
 	GError *error = NULL;
 	GOptionContext *context;
+	GnomeProgram *program;
+
+#ifdef ENABLE_NLS
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+#endif
 
 	context = g_option_context_new ("");
-	g_option_context_add_main_entries (context, entries, NULL);	/* FIXME - set translation domain */
+
+#ifdef HAVE_LIBGNOME_214
+	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
+	program = gnome_program_init (PACKAGE, VERSION,
+	                              LIBGNOMEUI_MODULE, argc, argv,
+	                              GNOME_PARAM_GOPTION_CONTEXT, context,
+	                              GNOME_PARAM_HUMAN_READABLE_NAME, _("IRC Chat"),
+	                              GNOME_PROGRAM_STANDARD_PROPERTIES,
+	                              NULL);
+#else
+	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_parse (context, &argc, &argv, &error);
+#endif
 
 	if (error) {
-		fprintf (stderr, "xchat-gnome: %s\nTry `xchat-gnome --help' for more information\n", error->message);
+		fprintf (stderr, _("xchat-gnome: %s\nTry `xchat-gnome --help' for more information\n"), error->message);
 		g_error_free (error);
 		return 0;
 	}
@@ -78,25 +107,24 @@ fe_args (int argc, char *argv[])
 		return 0;
 	}
 
-	if (opt_noauto)
-		auto_connect = 0;
-
 	if (opt_cfgdir)
 		xdir_fs = opt_cfgdir;
 
-	gnome_program_init ("xchat-gnome", VERSION, LIBGNOMEUI_MODULE, argc, argv, NULL);
-
-	/* FIXME: this is kind of a silly place to put this, but it seems to want to
-	   follow gnome_program_init */
-	gnome_window_icon_set_default_from_file (XCHATSHAREDIR "/xchat-gnome-small.png");
-	return 1;
+	return -1;
 }
 
 void
 fe_init (void)
 {
+	gchar *accel_map;
+	GConfClient *client;
+
 	gnome_vfs_init ();
 
+	client = gconf_client_get_default ();
+	gconf_client_add_dir (client, "/apps/xchat", GCONF_CLIENT_PRELOAD_NONE, NULL);
+	g_object_unref (client);
+
 	u = userlist_new ();
 	gui.quit = FALSE;
 	servlist_init ();
@@ -113,6 +141,7 @@ fe_init (void)
 	/* Force various window-related options to match our interaction model */
 	prefs.use_server_tab = TRUE;
 	prefs.notices_tabs = FALSE;
+	prefs.servernotice = TRUE;
 
 	/* If we don't have a specific DCC IP address, force get-from-server */
 	if (strlen (prefs.dcc_ip_str) == 0)
@@ -121,11 +150,16 @@ fe_init (void)
 	/* Don't allow the core to autoload plugins. We use our own
 	 * method for autoloading.
 	 */
-	skip_plugins = 1;
+	arg_skip_plugins = 1;
 
+	accel_map = g_strdup_printf ("%s%c%s", get_xdir_fs (), G_DIR_SEPARATOR, "accelerator_map");
+	if (g_file_test (accel_map, G_FILE_TEST_EXISTS))
+		gtk_accel_map_load (accel_map);
+	g_free (accel_map);
+
+#ifdef USE_PLUGIN
 	plugins_initialize ();
-	if (!opt_noplugins)
-		autoload_plugins ();
+#endif
 }
 
 void
@@ -145,7 +179,11 @@ fe_main (void)
 void
 fe_cleanup (void)
 {
-	/* FIXME: implement */
+	gchar *accel_map;
+
+	accel_map = g_strdup_printf ("%s%c%s", get_xdir_fs (), G_DIR_SEPARATOR, "accelerator_map");
+	gtk_accel_map_save (accel_map);
+	g_free (accel_map);
 }
 
 void
@@ -169,11 +207,21 @@ fe_timeout_remove (int tag)
 void
 fe_new_window (struct session *sess, int focus)
 {
-	text_gui_add_text_buffer (sess);
+	static gboolean loaded = FALSE;
+
+	gui.current_session = sess;
+	conversation_panel_add_session (CONVERSATION_PANEL (gui.conversation_panel), sess);
+	topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, sess->topic);
 	if (sess->type == SESS_SERVER)
 		navigation_tree_create_new_network_entry (gui.server_tree, sess);
 	else if (sess->type == SESS_CHANNEL || sess->type == SESS_DIALOG)
 		navigation_tree_create_new_channel_entry (gui.server_tree, sess);
+#ifdef USE_PLUGIN
+	if (!(opt_noplugins || loaded)) {
+		loaded = TRUE;
+		autoload_plugins ();
+	}
+#endif
 }
 
 void
@@ -230,13 +278,15 @@ fe_idle_add (void *func, void *data)
 void
 fe_set_topic (struct session *sess, char *topic)
 {
-	set_gui_topic (sess, topic);
+	topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, topic);
 }
 
 void
 fe_set_hilight (struct session *sess)
 {
 	navigation_model_set_hilight (gui.tree_model, sess);
+	if (!gtk_window_is_active (GTK_WINDOW (gui.main_window)))
+		gtk_window_set_urgency_hint (GTK_WINDOW (gui.main_window), TRUE);
 }
 
 void
@@ -309,7 +359,7 @@ fe_notify_update (char *name)
 void
 fe_text_clear (struct session *sess)
 {
-	clear_buffer (sess);
+	conversation_panel_clear (CONVERSATION_PANEL (gui.conversation_panel), sess);
 }
 
 void
@@ -322,7 +372,7 @@ fe_close_window (struct session *sess)
 			navigation_tree_remove_server (gui.server_tree, sess);
 	}
 
-	kill_session_callback (sess);
+	session_free (sess);
 }
 
 void
@@ -340,12 +390,12 @@ fe_progressbar_end (struct server *serv)
 void
 fe_print_text (struct session *sess, char *text)
 {
-	session_gui *tgui = (session_gui *) sess->gui;
-	if (tgui == NULL)
-		return;
-	text_gui_print (tgui->buffer, text, prefs.indent_nicks);
+	conversation_panel_print (CONVERSATION_PANEL (gui.conversation_panel), sess, text, prefs.indent_nicks);
 	sess->new_data = TRUE;
 	navigation_model_set_hilight (gui.tree_model, sess);
+	if (sess->nick_said)
+		if (!gtk_window_is_active (GTK_WINDOW (gui.main_window)))
+			gtk_window_set_urgency_hint (GTK_WINDOW (gui.main_window), TRUE);
 }
 
 void
@@ -357,7 +407,7 @@ fe_userlist_insert (struct session *sess, struct User *newuser, int row, int sel
 int
 fe_userlist_remove (struct session *sess, struct User *user)
 {
-	return userlist_remove (u, sess, user);
+	return userlist_remove_user (u, sess, user);
 }
 
 void
@@ -381,7 +431,7 @@ fe_userlist_numbers (struct session *sess)
 void
 fe_userlist_clear (struct session *sess)
 {
-	userlist_clear (u, sess);
+	userlist_clear_all (u, sess);
 }
 
 void
@@ -518,45 +568,17 @@ fe_beep (void)
 	gdk_beep ();
 }
 
-typedef struct
-{
-	session *sess;
-	unsigned char *sstr;
-} fe_lastlog_info;
-
-static void
-fe_lastlog_foreach (GtkXText * xtext, unsigned char *text, fe_lastlog_info * info)
-{
-	session_gui *tgui = (session_gui *) info->sess->gui;
-
-	if (nocasestrstr (text, info->sstr))
-		text_gui_print (tgui->buffer, text, prefs.indent_nicks);
-}
-
 void
 fe_lastlog (session * sess, session * lastlog_sess, char *sstr)
 {
-	session_gui *tgui = (session_gui *) sess->gui;
-	session_gui *lgui = (session_gui *) lastlog_sess->gui;
-	if (gtk_xtext_is_empty (tgui->buffer)) {
-		text_gui_print (lgui->buffer, _("Search buffer is empty.\n"), TRUE);
-	} else {
-		fe_lastlog_info info;
-		info.sess = lastlog_sess;
-		info.sstr = sstr;
-		gtk_xtext_foreach (tgui->buffer, (GtkXTextForeach) fe_lastlog_foreach, &info);
-	}
+	conversation_panel_lastlog (CONVERSATION_PANEL (gui.conversation_panel), sess, lastlog_sess, sstr);
 }
 
 void
-fe_set_lag (server * serv, int lag)
+fe_set_lag (server *serv, int lag)
 {
-	GSList *list = sess_list;
-	session *sess;
-	gdouble per;
-	char tip[64];
-	unsigned long nowtim;
-	session_gui *tgui;
+	unsigned long now;
+	float seconds;
 
 	if (gui.quit)
 		return;
@@ -564,69 +586,22 @@ fe_set_lag (server * serv, int lag)
 	if (lag == -1) {
 		if (!serv->lag_sent)
 			return;
-		nowtim = make_ping_time ();
-		lag = (nowtim - serv->lag_sent) / 100000;
+		now = make_ping_time ();
+		seconds = (now - serv->lag_sent) / 1000000.0f;
+	} else {
+		seconds = lag / 10.0f;
 	}
 
-	per = (double) ((double) lag / 40.0);
-	if (per > 1.0)
-		per = 1.0;
-	snprintf (tip, sizeof (tip) - 1, _("%s%d.%ds lag"), serv->lag_sent ? "+" : "", lag / 10, lag % 10);
-	while (list) {
-		sess = list->data;
-		if (sess->server == serv) {
-			tgui = (session_gui *) sess->gui;
-			if (tgui) {
-				tgui->lag_value = per;
-				if (tgui->lag_text)
-					free (tgui->lag_text);
-				tgui->lag_text = strdup (tip);
-			}
-		}
-		list = list->next;
-	}
-	if (serv == gui.current_session->server) {
-		set_statusbar ();
-	}
+	status_bar_set_lag (STATUS_BAR (gui.status_bar), serv, seconds, (serv->lag_sent != 0.0f));
 }
 
 void
 fe_set_throttle (server * serv)
 {
-	GSList *list = sess_list;
-	session *sess;
-	gdouble per;
-	char tip[64];
-	session_gui *tgui;
-
 	if (gui.quit)
 		return;
 
-	per = (gdouble) serv->sendq_len / 1024.0;
-	if (per > 1.0)
-		per = 1.0;
-
-	snprintf (tip, sizeof (tip) - 1, _("%d bytes buffered"), serv->sendq_len);
-	while (list) {
-		sess = list->data;
-		if (sess->server == serv) {
-			tgui = (session_gui *) sess->gui;
-			if (tgui) {
-				tgui->queue_value = per;
-				if (tgui->queue_text)
-					free (tgui->queue_text);
-				if (per != 0) {
-					tgui->queue_text = strdup (tip);
-				} else {
-					tgui->queue_text = NULL;
-				}
-			}
-		}
-		list = list->next;
-	}
-	if (serv == gui.current_session->server) {
-		set_statusbar ();
-	}
+	status_bar_set_queue (STATUS_BAR (gui.status_bar), serv, serv->sendq_len);
 }
 
 void
@@ -644,16 +619,14 @@ fe_serverlist_open (session * sess)
 void
 fe_ctrl_gui (session * sess, int action, int arg)
 {
-	GtkWidget *window = glade_xml_get_widget (gui.xml, "xchat-gnome");
-
 	switch (action) {
 	case 0:
-		gtk_widget_hide (window);
+		gtk_widget_hide (gui.main_window);
 		break;
 
 	case 1:
-		gtk_widget_show (window);
-		gtk_window_present (GTK_WINDOW (window));
+		gtk_widget_show (gui.main_window);
+		gtk_window_present (GTK_WINDOW (gui.main_window));
 		break;
 
 	case 2:
@@ -666,7 +639,7 @@ fe_ctrl_gui (session * sess, int action, int arg)
 		break;
 
 	case 5:
-		gtk_window_iconify (GTK_WINDOW (window));
+		gtk_window_iconify (GTK_WINDOW (gui.main_window));
 		break;
 	}
 }
@@ -679,13 +652,11 @@ fe_confirm (const char *message, void (*yesproc) (void *), void (*noproc) (void
 int
 fe_gui_info (session * sess, int info_type)
 {
-	GtkWidget *window = glade_xml_get_widget (gui.xml, "xchat-gnome");
-
 	switch (info_type) {
 	case 0:
-		if (!GTK_WIDGET_VISIBLE (GTK_WINDOW (window)))
+		if (!GTK_WIDGET_VISIBLE (GTK_WINDOW (gui.main_window)))
 			return 2;
-		if (gtk_window_is_active (GTK_WINDOW (window)))
+		if (gtk_window_is_active (GTK_WINDOW (gui.main_window)))
 			return 1;
 		return 0;
 		break;
@@ -694,6 +665,13 @@ fe_gui_info (session * sess, int info_type)
 	return -1;
 }
 
+void *
+fe_gui_info_ptr (session *sess, int info_type)
+{
+	return NULL;
+	/* FIXME implement? */
+}
+
 void
 fe_set_inputbox_cursor (session * sess, int delta, int pos)
 {
@@ -852,8 +830,8 @@ fe_open_url (const char *url)
 	}
 
 	if (err != NULL) {
-		gchar *message = g_strdup_printf ("Unable to show '%s'", url);
-		error_dialog (_(message), _(err->message));
+		gchar *message = g_strdup_printf (_("Unable to show '%s'"), url);
+		error_dialog (message, err->message);
 		g_free (message);
 		g_error_free (err);
 	}
@@ -882,3 +860,28 @@ fe_uselect (session *sess, char *word[], int do_clear, int scroll_to)
 {
 	/* FIXME: implement? */
 }
+
+void
+fe_server_event (server *serv, int type, int arg)
+{
+	/* FIXME: implement? */
+}
+
+void
+fe_userlist_set_selected (struct session *sess)
+{
+	/* FIXME: implement? */
+}
+
+void
+joind (int action, struct server *serv)
+{
+	switch (action) {
+	case 0:
+		/* FIXME: show join dialog(?) */
+		break;
+	case 1:
+		/* FIXME: hide join dialog(?) */
+		break;
+	};
+}
