@@ -1,7 +1,7 @@
 /*
  * main-window.h - main GUI window functions
  *
- * Copyright (C) 2004-2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,9 +24,9 @@
 #ifndef XCHAT_GNOME_MAIN_WINDOW_H
 #define XCHAT_GNOME_MAIN_WINDOW_H
 
-void initialize_main_window ();
-void run_main_window ();
-void rename_main_window (gchar *server, gchar *channel);
-void set_statusbar ();
+void initialize_main_window (void);
+void run_main_window        (void);
+void rename_main_window     (gchar *server, gchar *channel);
+void set_nickname           (struct server *serv, char *newnick);
 
 #endif
